@@ -20,6 +20,7 @@ use crate::{
     AleoAmount,
     DPCError,
     Network,
+    ProgramScheme,
 };
 use snarkvm_algorithms::{
     commitment_tree::CommitmentMerkleTree,
@@ -31,11 +32,12 @@ use snarkvm_gadgets::{
     bits::Boolean,
     traits::algorithms::{CRHGadget, SNARKVerifierGadget},
 };
-use snarkvm_utilities::{bytes::ToBytes, has_duplicates, rand::UniformRand, to_bytes};
+use snarkvm_parameters::{prelude::*, testnet1::*};
+use snarkvm_utilities::{has_duplicates, rand::UniformRand, to_bytes, FromBytes, ToBytes};
 
 use itertools::Itertools;
 use rand::{CryptoRng, Rng};
-use std::{marker::PhantomData, sync::Arc};
+use std::sync::Arc;
 
 pub mod inner_circuit;
 pub use inner_circuit::*;
@@ -57,9 +59,6 @@ pub use transaction::*;
 
 pub mod instantiated;
 
-#[cfg(test)]
-mod tests;
-
 ///////////////////////////////////////////////////////////////////////////////
 
 /// Trait that stores information about the testnet1 DPC scheme.
@@ -70,7 +69,7 @@ pub trait Testnet1Components: DPCComponents {
 
     /// Group and Model Parameters for record encryption
     type EncryptionGroup: Group + ProjectiveCurve;
-    type EncryptionModelParameters: MontgomeryParameters + TwistedEdwardsParameters;
+    type EncryptionParameters: MontgomeryParameters + TwistedEdwardsParameters;
 
     /// SNARK for non-proof-verification checks
     type InnerSNARK: SNARK<
@@ -107,81 +106,16 @@ pub trait Testnet1Components: DPCComponents {
 ///////////////////////////////////////////////////////////////////////////////
 
 pub struct DPC<C: Testnet1Components> {
-    _components: PhantomData<C>,
-}
-
-impl<C: Testnet1Components> DPC<C> {
-    pub fn generate_system_parameters<R: Rng + CryptoRng>(rng: &mut R) -> Result<SystemParameters<C>, DPCError> {
-        let time = start_timer!(|| "Account commitment scheme setup");
-        let account_commitment = C::AccountCommitment::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Account encryption scheme setup");
-        let account_encryption = <C::AccountEncryption as EncryptionScheme>::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Account signature setup");
-        let account_signature = C::AccountSignature::setup(rng)?;
-        end_timer!(time);
-
-        let time = start_timer!(|| "Encrypted record CRH setup");
-        let encrypted_record_crh = C::EncryptedRecordCRH::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Inner circuit ID CRH setup");
-        let inner_circuit_id_crh = C::InnerCircuitIDCRH::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Local data commitment setup");
-        let local_data_commitment = C::LocalDataCommitment::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Local data CRH setup");
-        let local_data_crh = C::LocalDataCRH::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Program verifying key CRH setup");
-        let program_verification_key_crh = C::ProgramVerificationKeyCRH::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Program verification key commitment setup");
-        let program_verification_key_commitment = C::ProgramVerificationKeyCommitment::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Record commitment scheme setup");
-        let record_commitment = C::RecordCommitment::setup(rng);
-        end_timer!(time);
-
-        let time = start_timer!(|| "Serial nonce CRH setup");
-        let serial_number_nonce = C::SerialNumberNonceCRH::setup(rng);
-        end_timer!(time);
-
-        Ok(SystemParameters {
-            account_commitment,
-            account_encryption,
-            account_signature,
-            encrypted_record_crh,
-            inner_circuit_id_crh,
-            local_data_crh,
-            local_data_commitment,
-            program_verification_key_commitment,
-            program_verification_key_crh,
-            record_commitment,
-            serial_number_nonce,
-        })
-    }
-
-    pub fn generate_noop_program_snark_parameters<R: Rng + CryptoRng>(
-        system_parameters: &SystemParameters<C>,
-        rng: &mut R,
-    ) -> Result<NoopProgramSNARKParameters<C>, DPCError> {
-        let (pk, pvk) = C::NoopProgramSNARK::setup(&NoopCircuit::blank(system_parameters), rng)?;
-
-        Ok(NoopProgramSNARKParameters {
-            proving_key: pk,
-            verifying_key: pvk.into(),
-        })
-    }
+    pub system_parameters: SystemParameters<C>,
+    pub noop_program: NoopProgram<C>,
+    pub inner_snark_parameters: (
+        Option<<C::InnerSNARK as SNARK>::ProvingKey>,
+        <C::InnerSNARK as SNARK>::PreparedVerifyingKey,
+    ),
+    pub outer_snark_parameters: (
+        Option<<C::OuterSNARK as SNARK>::ProvingKey>,
+        <C::OuterSNARK as SNARK>::PreparedVerifyingKey,
+    ),
 }
 
 impl<C: Testnet1Components, L: LedgerScheme> DPCScheme<L> for DPC<C>
@@ -196,35 +130,25 @@ where
     >,
 {
     type Account = Account<C>;
+    type Execution = Execution;
     type LocalData = LocalData<C>;
-    type NetworkParameters = PublicParameters<C>;
-    type Payload = <Self::Record as RecordScheme>::Payload;
-    type PrivateProgramInput = PrivateProgramInput;
     type Record = Record<C>;
     type SystemParameters = SystemParameters<C>;
     type Transaction = Transaction<C>;
     type TransactionKernel = TransactionKernel<C>;
 
-    fn setup<R: Rng + CryptoRng>(
-        ledger_parameters: &Arc<C::MerkleParameters>,
-        rng: &mut R,
-    ) -> anyhow::Result<Self::NetworkParameters> {
+    fn setup<R: Rng + CryptoRng>(ledger_parameters: &Arc<C::MerkleParameters>, rng: &mut R) -> anyhow::Result<Self> {
         let setup_time = start_timer!(|| "DPC::setup");
-        let system_parameters = Self::generate_system_parameters(rng)?;
+        let system_parameters = Self::SystemParameters::setup(rng)?;
 
-        let program_snark_setup_time = start_timer!(|| "Dummy program SNARK setup");
-        let noop_program_snark_parameters = Self::generate_noop_program_snark_parameters(&system_parameters, rng)?;
-        let program_snark_proof = C::NoopProgramSNARK::prove(
-            &noop_program_snark_parameters.proving_key,
-            &NoopCircuit::blank(&system_parameters),
+        let noop_program_timer = start_timer!(|| "Noop program SNARK setup");
+        let noop_program = NoopProgram::setup(
+            &system_parameters.local_data_commitment,
+            &system_parameters.program_verification_key_crh,
             rng,
         )?;
-        end_timer!(program_snark_setup_time);
-
-        let program_snark_vk_and_proof = PrivateProgramInput {
-            verifying_key: to_bytes![noop_program_snark_parameters.verifying_key]?,
-            proof: to_bytes![program_snark_proof]?,
-        };
+        let noop_program_execution = noop_program.execute_blank(rng)?;
+        end_timer!(noop_program_timer);
 
         let snark_setup_time = start_timer!(|| "Execute inner SNARK setup");
         let inner_circuit = InnerCircuit::blank(&system_parameters, ledger_parameters);
@@ -241,30 +165,69 @@ where
                 ledger_parameters.clone(),
                 inner_snark_vk,
                 inner_snark_proof,
-                program_snark_vk_and_proof,
+                noop_program_execution,
             ),
             rng,
         )?;
         end_timer!(snark_setup_time);
         end_timer!(setup_time);
 
-        Ok(PublicParameters {
+        Ok(Self {
             system_parameters,
-            noop_program_snark_parameters,
+            noop_program,
             inner_snark_parameters: (Some(inner_snark_parameters.0), inner_snark_parameters.1),
             outer_snark_parameters: (Some(outer_snark_parameters.0), outer_snark_parameters.1),
         })
     }
 
-    fn create_account<R: Rng + CryptoRng>(
-        parameters: &Self::SystemParameters,
-        rng: &mut R,
-    ) -> anyhow::Result<Self::Account> {
+    fn load(verify_only: bool) -> anyhow::Result<Self> {
+        let timer = start_timer!(|| "DPC::load");
+        let system_parameters = Self::SystemParameters::load()?;
+        let noop_program = NoopProgram::load(
+            &system_parameters.local_data_commitment,
+            &system_parameters.program_verification_key_crh,
+        )?;
+        let inner_snark_parameters = {
+            let inner_snark_pk = match verify_only {
+                true => None,
+                false => Some(<C::InnerSNARK as SNARK>::ProvingKey::read(
+                    InnerSNARKPKParameters::load_bytes()?.as_slice(),
+                )?),
+            };
+            let inner_snark_vk: <C::InnerSNARK as SNARK>::VerifyingKey =
+                <C::InnerSNARK as SNARK>::VerifyingKey::read(InnerSNARKVKParameters::load_bytes()?.as_slice())?;
+
+            (inner_snark_pk, inner_snark_vk.into())
+        };
+
+        let outer_snark_parameters = {
+            let outer_snark_pk = match verify_only {
+                true => None,
+                false => Some(<C::OuterSNARK as SNARK>::ProvingKey::read(
+                    OuterSNARKPKParameters::load_bytes()?.as_slice(),
+                )?),
+            };
+            let outer_snark_vk: <C::OuterSNARK as SNARK>::VerifyingKey =
+                <C::OuterSNARK as SNARK>::VerifyingKey::read(OuterSNARKVKParameters::load_bytes()?.as_slice())?;
+
+            (outer_snark_pk, outer_snark_vk.into())
+        };
+        end_timer!(timer);
+
+        Ok(Self {
+            system_parameters,
+            noop_program,
+            inner_snark_parameters,
+            outer_snark_parameters,
+        })
+    }
+
+    fn create_account<R: Rng + CryptoRng>(&self, rng: &mut R) -> anyhow::Result<Self::Account> {
         let time = start_timer!(|| "DPC::create_account");
         let account = Account::new(
-            &parameters.account_signature,
-            &parameters.account_commitment,
-            &parameters.account_encryption,
+            &self.system_parameters.account_signature,
+            &self.system_parameters.account_commitment,
+            &self.system_parameters.account_encryption,
             rng,
         )?;
         end_timer!(time);
@@ -272,7 +235,7 @@ where
     }
 
     fn execute_offline_phase<R: Rng + CryptoRng>(
-        parameters: Self::SystemParameters,
+        &self,
         old_private_keys: &Vec<<Self::Account as AccountScheme>::PrivateKey>,
         old_records: Vec<Self::Record>,
         new_records: Vec<Self::Record>,
@@ -298,7 +261,8 @@ where
                 value_balance = value_balance.add(AleoAmount::from_bytes(record.value() as i64));
             }
 
-            let (sn, randomizer) = record.to_serial_number(&parameters.account_signature, &old_private_keys[i])?;
+            let (sn, randomizer) =
+                record.to_serial_number(&self.system_parameters.account_signature, &old_private_keys[i])?;
             joint_serial_numbers.extend_from_slice(&to_bytes![sn]?);
             old_serial_numbers.push(sn);
             old_randomizers.push(randomizer);
@@ -349,7 +313,7 @@ where
 
             let commitment_randomness = <C::LocalDataCommitment as CommitmentScheme>::Randomness::rand(rng);
             let commitment = C::LocalDataCommitment::commit(
-                &parameters.local_data_commitment,
+                &self.system_parameters.local_data_commitment,
                 &input_bytes,
                 &commitment_randomness,
             )?;
@@ -364,7 +328,7 @@ where
 
             let commitment_randomness = <C::LocalDataCommitment as CommitmentScheme>::Randomness::rand(rng);
             let commitment = C::LocalDataCommitment::commit(
-                &parameters.local_data_commitment,
+                &self.system_parameters.local_data_commitment,
                 &input_bytes,
                 &commitment_randomness,
             )?;
@@ -379,7 +343,7 @@ where
             new_record_commitments[0].clone(),
             new_record_commitments[1].clone(),
         ];
-        let local_data_merkle_tree = CommitmentMerkleTree::new(parameters.local_data_crh.clone(), &leaves)?;
+        let local_data_merkle_tree = CommitmentMerkleTree::new(self.system_parameters.local_data_crh.clone(), &leaves)?;
 
         end_timer!(local_data_merkle_tree_timer);
 
@@ -394,7 +358,7 @@ where
             }
             let program_randomness = <C::ProgramVerificationKeyCommitment as CommitmentScheme>::Randomness::rand(rng);
             let program_commitment = C::ProgramVerificationKeyCommitment::commit(
-                &parameters.program_verification_key_commitment,
+                &self.system_parameters.program_verification_key_commitment,
                 &input,
                 &program_randomness,
             )?;
@@ -409,16 +373,15 @@ where
         let mut new_encrypted_records = Vec::with_capacity(C::NUM_OUTPUT_RECORDS);
 
         for record in &new_records {
-            let (encrypted_record, record_encryption_randomness) = EncryptedRecord::encrypt(&parameters, record, rng)?;
+            let (encrypted_record, record_encryption_randomness) =
+                EncryptedRecord::encrypt(&self.system_parameters, record, rng)?;
 
             new_records_encryption_randomness.push(record_encryption_randomness);
-            new_encrypted_record_hashes.push(encrypted_record.to_hash(&parameters)?);
+            new_encrypted_record_hashes.push(encrypted_record.to_hash(&self.system_parameters)?);
             new_encrypted_records.push(encrypted_record);
         }
 
         Ok(TransactionKernel {
-            system_parameters: parameters,
-
             old_records,
             old_serial_numbers,
             old_randomizers,
@@ -443,10 +406,10 @@ where
     }
 
     fn execute_online_phase<R: Rng + CryptoRng>(
-        parameters: &Self::NetworkParameters,
+        &self,
         old_private_keys: &Vec<<Self::Account as AccountScheme>::PrivateKey>,
         transaction_kernel: Self::TransactionKernel,
-        program_proofs: Vec<Self::PrivateProgramInput>,
+        program_proofs: Vec<Self::Execution>,
         ledger: &L,
         rng: &mut R,
     ) -> anyhow::Result<(Vec<Self::Record>, Self::Transaction)> {
@@ -456,8 +419,6 @@ where
         let exec_time = start_timer!(|| "DPC::execute_online_phase");
 
         let TransactionKernel {
-            system_parameters,
-
             old_records,
             old_serial_numbers,
             old_randomizers,
@@ -517,7 +478,7 @@ where
         for i in 0..C::NUM_INPUT_RECORDS {
             // Sign the transaction data
             let account_signature = C::AccountSignature::sign(
-                &system_parameters.account_signature,
+                &self.system_parameters.account_signature,
                 &old_private_keys[i].sk_sig,
                 &signature_message,
                 rng,
@@ -525,7 +486,7 @@ where
 
             // Randomize the signature
             let randomized_signature = C::AccountSignature::randomize_signature(
-                &system_parameters.account_signature,
+                &self.system_parameters.account_signature,
                 &account_signature,
                 &old_randomizers[i],
             )?;
@@ -541,7 +502,7 @@ where
 
         for (record, ciphertext_randomness) in new_records.iter().zip_eq(&new_records_encryption_randomness) {
             let record_encryption_gadget_components = EncryptedRecord::prepare_encryption_gadget_components(
-                &system_parameters,
+                &self.system_parameters,
                 &record,
                 ciphertext_randomness,
             )?;
@@ -551,7 +512,7 @@ where
 
         let inner_proof = {
             let circuit = InnerCircuit::new(
-                parameters.system_parameters.clone(),
+                self.system_parameters.clone(),
                 ledger.parameters().clone(),
                 ledger_digest.clone(),
                 old_records,
@@ -573,7 +534,7 @@ where
                 network_id,
             );
 
-            let inner_snark_parameters = match &parameters.inner_snark_parameters.0 {
+            let inner_snark_parameters = match &self.inner_snark_parameters.0 {
                 Some(inner_snark_parameters) => inner_snark_parameters,
                 None => return Err(DPCError::MissingInnerSnarkProvingParameters.into()),
             };
@@ -584,7 +545,7 @@ where
         // Verify that the inner proof passes
         {
             let input = InnerCircuitVerifierInput {
-                system_parameters: parameters.system_parameters.clone(),
+                system_parameters: self.system_parameters.clone(),
                 ledger_parameters: ledger.parameters().clone(),
                 ledger_digest: ledger_digest.clone(),
                 old_serial_numbers: old_serial_numbers.clone(),
@@ -598,22 +559,22 @@ where
             };
 
             assert!(C::InnerSNARK::verify(
-                &parameters.inner_snark_parameters.1,
+                &self.inner_snark_parameters.1,
                 &input,
                 &inner_proof
             )?);
         }
 
-        let inner_snark_vk: <C::InnerSNARK as SNARK>::VerifyingKey = parameters.inner_snark_parameters.1.clone().into();
+        let inner_snark_vk: <C::InnerSNARK as SNARK>::VerifyingKey = self.inner_snark_parameters.1.clone().into();
 
         let inner_circuit_id =
-            <C::InnerCircuitIDCRH as CRH>::hash(&parameters.system_parameters.inner_circuit_id_crh, &to_bytes![
+            <C::InnerCircuitIDCRH as CRH>::hash(&self.system_parameters.inner_circuit_id_crh, &to_bytes![
                 inner_snark_vk
             ]?)?;
 
         let transaction_proof = {
             let circuit = OuterCircuit::new(
-                parameters.system_parameters.clone(),
+                self.system_parameters.clone(),
                 ledger.parameters().clone(),
                 ledger_digest.clone(),
                 old_serial_numbers.clone(),
@@ -631,7 +592,7 @@ where
                 inner_circuit_id.clone(),
             );
 
-            let outer_snark_parameters = match &parameters.outer_snark_parameters.0 {
+            let outer_snark_parameters = match &self.outer_snark_parameters.0 {
                 Some(outer_snark_parameters) => outer_snark_parameters,
                 None => return Err(DPCError::MissingOuterSnarkProvingParameters.into()),
             };
@@ -659,7 +620,7 @@ where
         Ok((new_records, transaction))
     }
 
-    fn verify(parameters: &Self::NetworkParameters, transaction: &Self::Transaction, ledger: &L) -> bool {
+    fn verify(&self, transaction: &Self::Transaction, ledger: &L) -> bool {
         let verify_time = start_timer!(|| "DPC::verify");
 
         // Returns false if the number of serial numbers in the transaction is incorrect.
@@ -743,7 +704,7 @@ where
             }
         };
 
-        let account_signature = &parameters.system_parameters.account_signature;
+        let account_signature = &self.system_parameters.account_signature;
         for (pk, sig) in transaction.old_serial_numbers().iter().zip(transaction.signatures()) {
             match C::AccountSignature::verify(account_signature, pk, &signature_message, sig) {
                 Ok(is_valid) => {
@@ -771,7 +732,7 @@ where
 
         let mut new_encrypted_record_hashes = Vec::with_capacity(C::NUM_OUTPUT_RECORDS);
         for encrypted_record in transaction.encrypted_records() {
-            match encrypted_record.to_hash(&parameters.system_parameters) {
+            match encrypted_record.to_hash(&self.system_parameters) {
                 Ok(hash) => new_encrypted_record_hashes.push(hash),
                 _ => {
                     eprintln!("Unable to hash encrypted record.");
@@ -781,7 +742,7 @@ where
         }
 
         let inner_snark_input = InnerCircuitVerifierInput {
-            system_parameters: parameters.system_parameters.clone(),
+            system_parameters: self.system_parameters.clone(),
             ledger_parameters: ledger.parameters().clone(),
             ledger_digest: transaction.ledger_digest().clone(),
             old_serial_numbers: transaction.old_serial_numbers().to_vec(),
@@ -795,7 +756,7 @@ where
         };
 
         let inner_snark_vk: <<C as Testnet1Components>::InnerSNARK as SNARK>::VerifyingKey =
-            parameters.inner_snark_parameters.1.clone().into();
+            self.inner_snark_parameters.1.clone().into();
 
         let inner_snark_vk_bytes = match to_bytes![inner_snark_vk] {
             Ok(bytes) => bytes,
@@ -808,7 +769,7 @@ where
         let outer_snark_input = OuterCircuitVerifierInput {
             inner_snark_verifier_input: inner_snark_input,
             inner_circuit_id: match C::InnerCircuitIDCRH::hash(
-                &parameters.system_parameters.inner_circuit_id_crh,
+                &self.system_parameters.inner_circuit_id_crh,
                 &inner_snark_vk_bytes,
             ) {
                 Ok(hash) => hash,
@@ -820,7 +781,7 @@ where
         };
 
         match C::OuterSNARK::verify(
-            &parameters.outer_snark_parameters.1,
+            &self.outer_snark_parameters.1,
             &outer_snark_input,
             &transaction.transaction_proof,
         ) {
@@ -842,13 +803,9 @@ where
     }
 
     /// Returns true iff all the transactions in the block are valid according to the ledger.
-    fn verify_transactions(
-        parameters: &Self::NetworkParameters,
-        transactions: &[Self::Transaction],
-        ledger: &L,
-    ) -> bool {
+    fn verify_transactions(&self, transactions: &[Self::Transaction], ledger: &L) -> bool {
         for transaction in transactions {
-            if !Self::verify(parameters, transaction, ledger) {
+            if !self.verify(transaction, ledger) {
                 return false;
             }
         }
